@@ -67,4 +67,9 @@ class Combine1d(Combine):
         self.x_chunk_reindexed = [
             construct_member(groups) for groups in [np.array((1, 2, 3, 4))] * 4
         ]
-        self.kwargs = {"agg": flox.aggregations.mean, "axis": (3,)}
+        self.kwargs = {
+            "agg": flox.aggregations._initialize_aggregation(
+                "sum", "float64", np.float64, 0, None, {}
+            ),
+            "axis": (3,),
+        }
